@@ -1,6 +1,7 @@
 package operator
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"reflect"
@@ -25,7 +26,7 @@ const (
 	prestoTableFinalizer = metering.GroupName + "/prestotable"
 )
 
-func (op *Reporting) runPrestoTableWorker() {
+func (op *defaultReportingOperator) runPrestoTableWorker() {
 	logger := op.logger.WithField("component", "prestoTableWorker")
 	logger.Infof("PrestoTable worker started")
 	const maxRequeues = 10
@@ -33,7 +34,7 @@ func (op *Reporting) runPrestoTableWorker() {
 	}
 }
 
-func (op *Reporting) processPrestoTable(logger log.FieldLogger) bool {
+func (op *defaultReportingOperator) processPrestoTable(logger log.FieldLogger) bool {
 	obj, quit := op.prestoTableQueue.Get()
 	if quit {
 		logger.Infof("queue is shutting down, exiting PrestoTable worker")
@@ -50,7 +51,7 @@ func (op *Reporting) processPrestoTable(logger log.FieldLogger) bool {
 	return true
 }
 
-func (op *Reporting) syncPrestoTable(logger log.FieldLogger, key string) error {
+func (op *defaultReportingOperator) syncPrestoTable(logger log.FieldLogger, key string) error {
 	namespace, name, err := cache.SplitMetaNamespaceKey(key)
 	if err != nil {
 		logger.WithError(err).Errorf("invalid resource key :%s", key)
@@ -92,7 +93,7 @@ func (op *Reporting) syncPrestoTable(logger log.FieldLogger, key string) error {
 	return nil
 }
 
-func (op *Reporting) handlePrestoTable(logger log.FieldLogger, prestoTable *metering.PrestoTable) error {
+func (op *defaultReportingOperator) handlePrestoTable(logger log.FieldLogger, prestoTable *metering.PrestoTable) error {
 	if op.cfg.EnableFinalizers && prestoTableNeedsFinalizer(prestoTable) {
 		var err error
 		prestoTable, err = op.addPrestoTableFinalizer(prestoTable)
@@ -110,8 +111,8 @@ func (op *Reporting) handlePrestoTable(logger log.FieldLogger, prestoTable *mete
 	if prestoTable.Spec.TableName == "" {
 		return fmt.Errorf("spec.tableName must be set")
 	}
-	if len(prestoTable.Spec.Columns) == 0 {
-		return fmt.Errorf("spec.columns must be non-empty")
+	if !prestoTable.Spec.Unmanaged && len(prestoTable.Spec.Columns) == 0 {
+		return fmt.Errorf("spec.columns must be non-empty when spec.unmanaged is set to false")
 	}
 	if prestoTable.Spec.CreateTableAs && prestoTable.Spec.View {
 		return fmt.Errorf("spec.createTableAs and spec.view are mutually exclusive")
@@ -122,12 +123,20 @@ func (op *Reporting) handlePrestoTable(logger log.FieldLogger, prestoTable *mete
 		return nil
 	}
 
-	var needsUpdate bool
+	var (
+		needsUpdate bool
+		err         error
+	)
 	if prestoTable.Spec.Unmanaged {
 		logger.Infof("PrestoTable %s is unmanaged", prestoTable.Name)
+
+		prestoTable.Spec.Columns, err = op.prestoTableManager.QueryMetadata(prestoTable.Spec.Catalog, prestoTable.Spec.Schema, prestoTable.Spec.TableName)
+		if err != nil {
+			return fmt.Errorf("failed to query the %s Presto table metadata: %v", prestoTable.Spec.TableName, err)
+		}
+
 		needsUpdate = copyPrestoTableSpecToStatus(prestoTable)
 	} else {
-		var err error
 		var tableStr string
 		switch {
 		case prestoTable.Spec.View:
@@ -154,7 +163,7 @@ func (op *Reporting) handlePrestoTable(logger log.FieldLogger, prestoTable *mete
 
 	if needsUpdate {
 		var err error
-		prestoTable, err = op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(prestoTable)
+		prestoTable, err = op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(context.TODO(), prestoTable, metav1.UpdateOptions{})
 		if err != nil {
 			return fmt.Errorf("unable to update PrestoTable %s status: %s", prestoTable.Name, err)
 		}
@@ -217,9 +226,9 @@ func copyPrestoTableSpecToStatus(prestoTable *metering.PrestoTable) bool {
 	return needsUpdate
 }
 
-func (op *Reporting) addPrestoTableFinalizer(prestoTable *metering.PrestoTable) (*metering.PrestoTable, error) {
+func (op *defaultReportingOperator) addPrestoTableFinalizer(prestoTable *metering.PrestoTable) (*metering.PrestoTable, error) {
 	prestoTable.Finalizers = append(prestoTable.Finalizers, prestoTableFinalizer)
-	newPrestoTable, err := op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(prestoTable)
+	newPrestoTable, err := op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(context.TODO(), prestoTable, metav1.UpdateOptions{})
 	logger := op.logger.WithFields(log.Fields{"prestoTable": prestoTable.Name, "namespace": prestoTable.Namespace})
 	if err != nil {
 		logger.WithError(err).Errorf("error adding %s finalizer to PrestoTable: %s/%s", prestoTableFinalizer, prestoTable.Namespace, prestoTable.Name)
@@ -229,12 +238,12 @@ func (op *Reporting) addPrestoTableFinalizer(prestoTable *metering.PrestoTable)
 	return newPrestoTable, nil
 }
 
-func (op *Reporting) removePrestoTableFinalizer(prestoTable *metering.PrestoTable) (*metering.PrestoTable, error) {
+func (op *defaultReportingOperator) removePrestoTableFinalizer(prestoTable *metering.PrestoTable) (*metering.PrestoTable, error) {
 	if !slice.ContainsString(prestoTable.ObjectMeta.Finalizers, prestoTableFinalizer, nil) {
 		return prestoTable, nil
 	}
 	prestoTable.Finalizers = slice.RemoveString(prestoTable.Finalizers, prestoTableFinalizer, nil)
-	newPrestoTable, err := op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(prestoTable)
+	newPrestoTable, err := op.meteringClient.MeteringV1().PrestoTables(prestoTable.Namespace).Update(context.TODO(), prestoTable, metav1.UpdateOptions{})
 	logger := op.logger.WithFields(log.Fields{"prestoTable": prestoTable.Name, "namespace": prestoTable.Namespace})
 	if err != nil {
 		logger.WithError(err).Errorf("error removing %s finalizer from PrestoTable: %s/%s", prestoTableFinalizer, prestoTable.Namespace, prestoTable.Name)
@@ -248,16 +257,15 @@ func prestoTableNeedsFinalizer(prestoTable *metering.PrestoTable) bool {
 	return prestoTable.ObjectMeta.DeletionTimestamp == nil && !slice.ContainsString(prestoTable.ObjectMeta.Finalizers, prestoTableFinalizer, nil)
 }
 
-func (op *Reporting) dropPrestoTable(prestoTable *metering.PrestoTable) error {
+func (op *defaultReportingOperator) dropPrestoTable(prestoTable *metering.PrestoTable) error {
 	if !prestoTable.Spec.Unmanaged {
 		if prestoTable.Status.TableName == "" {
 			return nil
 		}
 		if prestoTable.Status.View {
 			return op.prestoTableManager.DropView(prestoTable.Status.Catalog, prestoTable.Status.Schema, prestoTable.Status.TableName, true)
-		} else {
-			return op.prestoTableManager.DropTable(prestoTable.Status.Catalog, prestoTable.Status.Schema, prestoTable.Status.TableName, true)
 		}
+		return op.prestoTableManager.DropTable(prestoTable.Status.Catalog, prestoTable.Status.Schema, prestoTable.Status.TableName, true)
 	}
 	if prestoTable.Spec.Unmanaged {
 		return errors.New("cannot drop unmanaged PrestoTable")
@@ -265,7 +273,7 @@ func (op *Reporting) dropPrestoTable(prestoTable *metering.PrestoTable) error {
 	return errors.New("dropping PrestoTables is currently unsupported")
 }
 
-func (op *Reporting) createPrestoTableCR(obj metav1.Object, gvk schema.GroupVersionKind, catalog, schema, tableName string, columns []presto.Column, unmanaged, view bool, query string) (*metering.PrestoTable, error) {
+func (op *defaultReportingOperator) createPrestoTableCR(obj metav1.Object, gvk schema.GroupVersionKind, catalog, schema, tableName string, columns []presto.Column, unmanaged, view bool, query string) (*metering.PrestoTable, error) {
 	apiVersion := gvk.GroupVersion().String()
 	kind := gvk.Kind
 	name := obj.GetName()
@@ -304,7 +312,7 @@ func (op *Reporting) createPrestoTableCR(obj metav1.Object, gvk schema.GroupVers
 		},
 	}
 	var err error
-	prestoTable, err := op.meteringClient.MeteringV1().PrestoTables(namespace).Create(newPrestoTable)
+	prestoTable, err := op.meteringClient.MeteringV1().PrestoTables(namespace).Create(context.TODO(), newPrestoTable, metav1.CreateOptions{})
 	switch {
 	case apierrors.IsAlreadyExists(err):
 		op.logger.Warnf("PrestoTable %s already exists", resourceName)
@@ -318,11 +326,11 @@ func (op *Reporting) createPrestoTableCR(obj metav1.Object, gvk schema.GroupVers
 	return prestoTable, nil
 }
 
-func (op *Reporting) waitForPrestoTable(namespace, name string, pollInterval, timeout time.Duration) (*metering.PrestoTable, error) {
+func (op *defaultReportingOperator) waitForPrestoTable(namespace, name string, pollInterval, timeout time.Duration) (*metering.PrestoTable, error) {
 	var prestoTable *metering.PrestoTable
 	err := wait.Poll(pollInterval, timeout, func() (bool, error) {
 		var err error
-		prestoTable, err = op.meteringClient.MeteringV1().PrestoTables(namespace).Get(name, metav1.GetOptions{})
+		prestoTable, err = op.meteringClient.MeteringV1().PrestoTables(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 		if apierrors.IsNotFound(err) {
 			return false, nil
 		}
@@ -343,7 +351,7 @@ func (op *Reporting) waitForPrestoTable(namespace, name string, pollInterval, ti
 	return prestoTable, nil
 }
 
-func (op *Reporting) queueDependentsOfPrestoTable(prestoTable *metering.PrestoTable) error {
+func (op *defaultReportingOperator) queueDependentsOfPrestoTable(prestoTable *metering.PrestoTable) error {
 	reports, err := op.reportLister.Reports(prestoTable.Namespace).List(labels.Everything())
 	if err != nil {
 		return err
